# Copyright 2020 NXP
# SPDX-License-Identifier: BSD-3-Clause
OBJ_CLASSIFICATION = {'image': "cat.jpg",
                      'labels': "labels_mobilenet_quant_v1_224.txt",
                      'model': "mobilenet_v1_1.0_224_quant.tflite",
                      'sha1': "119f412b44429ee54643fd61da54530d844251ca",
                      'src': {'drive': "https://drive.google.com/file/d/"
                                       "1hO9xrXglzoEy0Tv6RsM9YtttTsXNyHws",
                              'github': "https://github.com/NXP/"
                                        "nxp-demo-experience-assets/releases/download/5.10.52_2.1.0/"
                                        "eIQObjectClassificationTFLite.zip"},
                      'window_title': "PyeIQ - Object Classification"}






# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

import cv2

from eiq.utils import Timer


class OpenCVDNN:
    def __init__(self, caffe=None, proto=None):
        self.blob = None
        self.det = None
        self.net = None
        self.inference_time = None

        if caffe and proto:
            self.net = cv2.dnn.readNetFromCaffe(proto, caffe)

    def create_blob(self, image, dims, scale, normalize):
        self.blob = cv2.dnn.blobFromImage(image, scale, (dims, dims),
                                          (normalize, normalize,
                                           normalize), False)

    def run_inference(self):
        timer = Timer()
        self.net.setInput(self.blob)

        with timer.timeit():
            self.det = self.net.forward()

        self.inference_time = timer.time



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

import os
import re
import subprocess

from eiq.apps.switch_image.config import LIB_PATH, LIBNN, RUN_LABEL_IMAGE, REGEX_GET_STRING, REGEX_GET_INTEGER_FLOAT


def run(use_accel, binary, model, image, labels):
    if use_accel:
        accel = 1
    else:
        accel = 0

    run_li = RUN_LABEL_IMAGE.format(binary, model, image, labels, accel)
    return subprocess.check_output(run_li, shell=True, stderr=subprocess.STDOUT)


def get_chances(line):
    x = re.findall(REGEX_GET_INTEGER_FLOAT, line)
    y = re.sub(REGEX_GET_STRING, u'', line, flags=re.UNICODE)
    y = y.lstrip()
    x.append(y)
    return x


def parser_cpu_gpu(data, include_accel):
    parsed_data = []
    res = [string.strip() for string in data.splitlines()]

    model_name = res[0].rsplit('/', 1)[1]
    parsed_data.append(model_name)

    if include_accel:
        index = 4
        start_index = 5
    else:
        index = 3
        start_index = 4

    average_time = res[index].rsplit(':', 1)[1]
    average_time = average_time.rsplit(' ', 1)[0]
    parsed_data.append(average_time)
        
    for i in range(start_index, len(res)):
        parsed_data.append(get_chances(res[i]))
    return parsed_data


def create_link():
    if not (LIBNN in os.listdir(LIB_PATH)):
        for lib in os.listdir(LIB_PATH):
            if LIBNN in lib:
                os.system(f"ln -s {os.path.join(LIB_PATH, lib)} {os.path.join(LIB_PATH, LIBNN)}")
                return

def run_label_image(accel, binary, model, image, labels):
    to_be_parsed = run(accel, binary, model, image, labels).decode('utf-8')
    return parser_cpu_gpu(to_be_parsed, accel)



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

import os

import cv2
import numpy as np

from eiq.config import FONT, INF_TIME_MSG, MODEL_MSG, QUANT_MSG0, QUANT_MSG1, SRC_MSG, FPS_MSG

class OpenCVOverlay:
    def __init__(self):
        self.time = None
        self.fps = None

    def draw_fps(self, frame, fps):
        fps_msg = "{}: {}".format(FPS_MSG, fps)
        x_offset = frame.shape[1] - (cv2.getTextSize(fps_msg, FONT['hershey'],
                                                     0.8, 2)[0][0] + 10)
        cv2.putText(frame, fps_msg,
                    (x_offset, 25), FONT['hershey'], 0.8,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, fps_msg,
                    (x_offset, 25), FONT['hershey'], 0.8,
                    FONT['color']['white'], 1, cv2.LINE_AA)

    def draw_info(self, frame, model, src, time, quant=None):
        model = os.path.basename(model)

        cv2.putText(frame, "{}: {}".format(INF_TIME_MSG, time),
                    (3, 20), FONT['hershey'], 0.5,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, "{}: {}".format(INF_TIME_MSG, time),
                    (3, 20), FONT['hershey'], 0.5,
                    FONT['color']['white'], 1, cv2.LINE_AA)

        y_offset = frame.shape[0] - cv2.getTextSize(src, FONT['hershey'],
                                                    0.5, 2)[0][1]
        cv2.putText(frame, "{}: {}".format(SRC_MSG, src),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, "{}: {}".format(SRC_MSG, src),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['white'], 1, cv2.LINE_AA)

        y_offset -= (cv2.getTextSize(model, FONT['hershey'], 0.5, 2)[0][1] + 3)
        cv2.putText(frame, "{}: {}".format(MODEL_MSG, model),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['black'], 2, cv2.LINE_AA)
        cv2.putText(frame, "{}: {}".format(MODEL_MSG, model),
                    (3, y_offset), FONT['hershey'], 0.5,
                    FONT['color']['white'], 1, cv2.LINE_AA)

        if not quant or (quant[0] == 0):
            y_offset -= (cv2.getTextSize(QUANT_MSG1, FONT['hershey'], 0.6, 2)[0][1] + 3) + 5
            cv2.putText(frame, QUANT_MSG1, (3, y_offset), FONT['hershey'], 0.6,
                        FONT['color']['black'], 2, cv2.LINE_AA)
            cv2.putText(frame, QUANT_MSG1, (3, y_offset), FONT['hershey'], 0.6,
                        FONT['color']['red'], 1, cv2.LINE_AA)

            y_offset -= (cv2.getTextSize(QUANT_MSG0, FONT['hershey'], 0.6, 2)[0][1] + 3)
            cv2.putText(frame, QUANT_MSG0, (3, y_offset), FONT['hershey'], 0.6,
                        FONT['color']['black'], 2, cv2.LINE_AA)
            cv2.putText(frame, QUANT_MSG0, (3, y_offset), FONT['hershey'], 0.6,
                        FONT['color']['red'], 1, cv2.LINE_AA)



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

import os

import cv2
import numpy as np
from PIL import Image

from eiq.config import FONT
from eiq.engines.tflite.inference import TFLiteInterpreter
from eiq.modules.detection.config import FACE_EYES_DETECTION
from eiq.modules.utils import DemoBase
from eiq.utils import Timer


class eIQFaceAndEyesDetection(DemoBase):
    def __init__(self, args=None):
        super().__init__(args, self.__class__.__name__, FACE_EYES_DETECTION)
        self.timer = Timer()

        self.eye_cascade = None
        self.face_cascade = None

    @staticmethod
    def description():
        return ("This demo uses:\n   * OpenCV as inference engine."
                "\n   * Haar Cascade as default algorithm.\n")

    def usage(self, name=None, labels=False, model=False):
        super().usage(name=name, labels=labels, model=model)

    def gather_data(self):
        super().gather_data()

        self.eye_cascade = os.path.join(self.model_dir,
                                        self.data['eye_cascade'])
        self.face_cascade = os.path.join(self.model_dir,
                                         self.data['face_cascade'])

    def detect_face(self, frame):
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

        with self.timer.timeit():
            faces = self.face_cascade.detectMultiScale(gray, 1.3, 5)

        for (x, y, w, h) in faces:
            cv2.rectangle(frame, (x, y), (x+w, y+h), (255, 0, 0), 2)
            roi_gray = gray[y:y+h, x:x+w]
            roi_color = frame[y:y+h, x:x+w]

            eyes = self.eye_cascade.detectMultiScale(roi_gray)
            for (ex, ey, ew, eh) in eyes:
                cv2.rectangle(roi_color, (ex, ey), (ex+ew, ey+eh),
                              (0, 255, 0), 2)

        self.overlay.draw_info(frame, "haarcascade_frontalface_default.xml",
                               self.media_src, self.timer.time)

        return frame

    def start(self):
        self.gather_data()
        self.validate_data(self.image, self.model,
                           self.eye_cascade, self.face_cascade)
        self.eye_cascade = cv2.CascadeClassifier(self.eye_cascade)
        self.face_cascade = cv2.CascadeClassifier(self.face_cascade)

    def run(self):
        self.start()
        self.run_inference(self.detect_face)



# Copyright 2018 The TensorFlow Authors
# Copyright 2021 NXP
#
# This file was copied from TensorFlow respecting its rights. All the modified
# parts below are according to TensorFlow's LICENSE terms.
#
# SPDX-License-Identifier:    Apache-2.0

import collections
import colorsys
import os
import random
import sys
import time

import gi
gi.require_version('Gst', '1.0')
from gi.repository import Gst

import cv2
import numpy as np
from PIL import Image

from eiq.config import FONT
from eiq.engines.opencv.inference import OpenCVDNN
from eiq.engines.tflite.inference import TFLiteInterpreter
from eiq.modules.detection.config import OBJ_DETECTION, OBJ_DETECTION_GST
from eiq.modules.utils import DemoBase
from eiq.multimedia import gstreamer

try:
    import svgwrite
    has_svgwrite = True
except ImportError:
    has_svgwrite = False


class eIQObjectDetection(DemoBase):
    def __init__(self, args=None):
        super().__init__(args, self.__class__.__name__, OBJ_DETECTION)

        self.colors = None

    @staticmethod
    def description():
        return ("This demo uses:\n   * TensorFlow Lite as inference engine."
                "\n   * Single Shot Detection as default algorithm.\n")

    def usage(self, name=None, labels=True, model=True):
        super().usage(name=name, labels=labels, model=model)

    def generate_colors(self):
        hsv_tuples = [(x / len(self.labels), 1., 1.)
                      for x in range(len(self.labels))]

        colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
        colors = list(map(lambda x: (int(x[0] * 255), int(x[1] * 255),
                                     int(x[2] * 255)), colors))
        random.seed(10101)
        random.shuffle(colors)
        random.seed(None)

        self.colors = colors

    def process_image(self, image):
        self.interpreter.set_tensor(np.expand_dims(image, axis=0))
        self.interpreter.run_inference()

        positions = self.interpreter.get_tensor(0, squeeze=True)
        classes = self.interpreter.get_tensor(1, squeeze=True)
        scores = self.interpreter.get_tensor(2, squeeze=True)

        result = []
        for idx, score in enumerate(scores):
            if score > 0.5:
                result.append({'pos': positions[idx], '_id': classes[idx]})
        return result

    def display_result(self, frame, result):
        width = frame.shape[1]
        height = frame.shape[0]

        for obj in result:
            pos = obj['pos']
            _id = obj['_id']

            x1 = int(pos[1] * width)
            x2 = int(pos[3] * width)
            y1 = int(pos[0] * height)
            y2 = int(pos[2] * height)

            top = max(0, np.floor(y1 + 0.5).astype('int32'))
            left = max(0, np.floor(x1 + 0.5).astype('int32'))
            bottom = min(height, np.floor(y2 + 0.5).astype('int32'))
            right = min(width, np.floor(x2 + 0.5).astype('int32'))

            label_size = cv2.getTextSize(self.labels[_id], FONT['hershey'],
                                         FONT['size'], FONT['thickness'])[0]
            label_rect_left = int(left - 3)
            label_rect_top = int(top - 3)
            label_rect_right = int(left + 3 + label_size[0])
            label_rect_bottom = int(top - 5 - label_size[1])

            cv2.rectangle(frame, (left, top), (right, bottom),
                          self.colors[int(_id) % len(self.colors)], 6)
            cv2.rectangle(frame, (label_rect_left, label_rect_top),
                          (label_rect_right, label_rect_bottom),
                          self.colors[int(_id) % len(self.colors)], -1)
            cv2.putText(frame, self.labels[_id], (left, int(top - 4)),
                        FONT['hershey'], FONT['size'],
                        FONT['color']['black'],
                        FONT['thickness'])
            self.overlay.draw_info(frame, self.model, self.media_src,
                                   self.interpreter.inference_time,
                                   self.interpreter.input_details[0]['quantization'])

    def detect_object(self, frame):
        image = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
        image = image.resize((self.interpreter.width(), self.interpreter.height()))
        top_result = self.process_image(image)
        self.display_result(frame, top_result)

        return frame

    def start(self):
        self.gather_data()
        self.validate_data(self.image, self.labels, self.model)
        self.interpreter = TFLiteInterpreter(self.model)
        self.labels = self.load_labels(self.labels)
        self.generate_colors()

    def run(self):
        self.start()
        self.run_inference(self.detect_object)



# Copyright 2019 Google LLC
# Copyright 2021 NXP
#
# This file was copied from Google LLC respecting its rights. All the modified
# parts below are according to Google LLC's LICENSE terms.
#
# SPDX-License-Identifier:    Apache-2.0

import collections
import sys

try:
    import svgwrite
except ImportError:
    None

import threading

import gi
gi.require_version('Gst', '1.0')
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, GObject, Gst, Gtk

import numpy as np


Object = collections.namedtuple('Object', ['id', 'score', 'bbox'])

GObject.threads_init()
Gst.init(None)


class BBox(collections.namedtuple('BBox', ['xmin', 'ymin', 'xmax', 'ymax'])):
    __slots__ = ()


def set_appsink_video_pipeline(device,width,height,
                        leaky="leaky=2 max-size-buffers=2"):

    return (("filesrc location={} ! qtdemux name=d d.video_0 ! \
                queue ! decodebin ! videorate ! videoconvert ! \
                videoscale n-threads=4 method=nearest-neighbour ! \
                video/x-raw,format=RGB,width={},height={} ! \
                queue {} ! appsink name=sink emit-signals=True"
            ).format(device,width,height,
                    leaky))

def set_appsink_pipeline(device,width,height,
                        leaky="leaky=2 max-size-buffers=2"):

    return (("v4l2src device={} do-timestamp=True ! videoconvert ! \
                videoscale n-threads=4 method=nearest-neighbour ! \
                video/x-raw,format=RGB,width={},height={} ! \
                queue {} ! appsink name=sink emit-signals=True"
            ).format(device,width,height,
                    leaky))

def set_appsrc_pipeline(width, height):
    return (("appsrc name=src is-live=True block=True ! \
                video/x-raw,format=RGB,width={},height={}, \
                framerate=30/1,interlace-mode=(string)progressive ! \
                videoconvert ! imxvideoconvert_g2d ! waylandsink"
            ).format(width, height))



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

LEFT = 0
TOP = 1
RIGHT = 2
BOTTOM = 3
CONFIDENCE = 4
CLASSES = 5

OBJ_DETECTION = {'image': "bus.jpg",
                 'labels': "labels_ssd_mobilenet_v1.txt",
                 'model': "ssd_mobilenet_v1_1_default_1.tflite",
                 'sha1': "eef8713a6507850998e9e405f0a2b02a79fba757",
                 'src': {'drive': "https://drive.google.com/file/d/"
                                  "1qwpKsDiKYttEqvoOPWgY8d8FSIyFgNGR",
                         'github': "https://github.com/NXP/"
                                   "nxp-demo-experience-assets/releases/download/5.10.52_2.1.0/"
                                   "eIQObjectDetection.zip"},
                 'window_title': "PyeIQ - Object Detection"}

OBJ_DETECTION_GST = {'labels': "coco_labels.txt",
                     'model': "mobilenet_ssd_v2_coco_quant_postprocess.tflite",
                     'sha1': "4736e758d8d626047df7cd1b3c38c72e77fd32ee",
                     'src': {'drive': "https://drive.google.com/file/d/"
                                      "1kQj5Lj8fS3BUMm2NyABLo9nmS6XonjZ0",
                             'github': "https://github.com/lizeze-0515/"
                                       "pymodel/releases/download/model_3.0.0/"
                                       "eIQObjectDetectionCVGST.zip"},
                     'window_title': "PyeIQ - Object Detection OpenCV"}
                     
OBJ_COVID19 = { 'image': "demo5.jpg",
                'model_distance':"mobilenet_ssd_v2_coco_quant_postprocess.tflite",
                'model_facemask': "facemask_int8.tflite",
                'priors': "priors.txt",
                'sha1': "7a5b25a8664b7e1c6c0e8b52202d1ea0c6138858",
                'src': {'drive': "https://drive.google.com/file/d/"
                                 "1EAI_V5aueSyADzhPlPNEmEKyAX8T7ZQK",
                        'github':"https://github.com/NXP/"
                                 "nxp-demo-experience-assets/releases/download/5.10.52_2.1.0/"
                                 "eIQObjectDetectionCOVID19.zip"},
                'window_title': "PyeIQ - Object Detection COVID19"}



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import print_function
from argparse import ArgumentParser
from contextlib import contextmanager
from datetime import timedelta

import logging

logging.basicConfig(level=logging.INFO)

from hashlib import sha1
import os
import pathlib
import requests
import shutil
from subprocess import PIPE, Popen
import sys
import tempfile
import collections
from time import monotonic
from urllib.error import URLError
from urllib.parse import urlparse
import urllib.request

from eiq.config import ID, MAX_TIME, TMP_FILE_PATH
from eiq.helper.google_drive_downloader import GoogleDriveDownloader as gdd


class Downloader:
    def __init__(self, args):
        self.args = args
        self.downloaded_file = None

    @staticmethod
    def check_servers(url_dict):
        elapsed = {}
        min_time = MAX_TIME

        for key, val in url_dict.items():
            try:
                e_time = requests.get(val).elapsed
                elapsed[e_time] = key
            except:
                pass

        for e_time in elapsed:
            min_time = min(min_time, e_time)

        if min_time == MAX_TIME:
            return None

        return elapsed[min_time]

    @staticmethod
    def download_from_url(url, filename=None, download_path=None):
        timer = Timer()

        try:
            log("Downloading '{0}'".format(filename))
            with timer.timeit():
                urllib.request.urlretrieve(url, download_path)
        except URLError as e:
            sys.exit("Something went wrong: {}".format(e))

    def download_from_web(self, url, filename=None,
                          download_path=None, drive=False):
        if filename is None:
            filename_parsed = urlparse(url)
            filename = os.path.basename(filename_parsed.path)

        if download_path is None:
            download_path = get_temporary_path(TMP_FILE_PATH)

        try:
            pathlib.Path(download_path).mkdir(parents=True, exist_ok=True)
        except:
            sys.exit("Path().mkdir() has failed "
                     "trying to create: {}".format(download_path))

        download_path = os.path.join(download_path, filename)

        if not (os.path.exists(download_path)):
            if not drive:
                self.download_from_url(url, filename, download_path)
            else:
                try:
                    gdd.download_file_from_google_drive(file_id=url,
                                                        dest_path=download_path)
                    self.downloaded_file = download_path
                except:
                    sys.exit("\n"
                             "Google Drive server could not be reached."
                             "Your download has been canceled.\n"
                             "Please try other download parameters, github or wget.\n"
                             "Exiting...")

        self.downloaded_file = download_path

    def retrieve_data(self, url_dict, filename=None, download_path=None,
                      sha1_hash=None, unzip=False):
        if os.path.exists(os.path.join(download_path, filename)):
            return

        drive_flag = False
        if self.args.download is not None:
            if self.args.download == 'wget':
                self.wget(url_dict['github'], filename, download_path)
                return
            
            try:
                url = url_dict[self.args.download]
            except:
                sys.exit("Your download parameter is invalid. Exiting...")
            
            if self.args.download == 'github':
                self.wget(url_dict['github'], filename, download_path)
                return
            
            if self.args.download == 'drive':
                sys.exit("Google driver is no longer supported in future release. Please use github instead. Exiting...")
        else:           
            self.wget(url_dict['github'], filename, download_path)
            return

        self.download_from_web(url, filename, download_path, drive=drive_flag)
        if unzip and self.downloaded_file is not None:
            if sha1_hash and check_sha1(self.downloaded_file, sha1_hash):
                shutil.unpack_archive(self.downloaded_file, download_path)
            else:
                os.remove(self.downloaded_file)
                sys.exit("The checksum of your file failed!"
                         "Your file is corrupted.\nRemoving and exiting...")

    @staticmethod
    def wget(url, filename, download_path):
        newfile = os.path.join(download_path, filename)

        try:
            pathlib.Path(download_path).mkdir(parents=True, exist_ok=True)
        except:
            sys.exit("Path().mkdir() has failed "
                     "trying to create: {}".format(download_path))

        Popen(["wget", "{}".format(url), "-O", "{}".format(newfile)]).wait()
        shutil.unpack_archive(newfile, download_path)


def log(*args):
    logging.info(" ".join("{}".format(a) for a in args))


class Framerate:
    def __init__(self):
        self.fps = 0

    @contextmanager
    def fpsit(self):
        window = collections.deque(maxlen=30)
        begin = monotonic()
        try:
            yield
        finally:
            end = monotonic()
            window.append(end - begin)
            self.fps = len(window) / sum(window)


class Timer:
    def __init__(self):
        self.time = 0

    @contextmanager
    def timeit(self):
        begin = monotonic()
        try:
            yield
        finally:
            end = monotonic()
            self.convert(end - begin)

    def convert(self, elapsed):
        self.time = str(timedelta(seconds=elapsed))


def get_temporary_path(*path):
    return os.path.join(tempfile.gettempdir(), *path)


def colored(msg, color):
    color_id = 0
    if color == "red":
        color_id = 31
    elif color == "green":
        color_id = 32
    elif color == "blue":
        color_id = 34
    elif color == "yellow":
        color_id = 33

    return "\033[{}m".format(color_id) + msg + "\033[0m"


def check_sha1(file_path, sha1_hash):
    with open(file_path, 'rb') as f:
        file = f.read()

    return sha1(file).hexdigest() == sha1_hash


def check_data(zipped_file, sha1sum, *data):
    for file in data:
        if file and not os.path.isfile(file):
            if os.path.isfile(zipped_file) and check_sha1(zipped_file, sha1sum):
                path = os.path.dirname(zipped_file)
                shutil.unpack_archive(zipped_file, path)
            else:
                return False
    return True


def file_type(file_path):
    if os.path.isfile(file_path):
        try:
            proc = Popen(['file', '-bi', '{}'.format(file_path)], stdout=PIPE)
            stdout = proc.communicate()[0].decode('ascii')
            stdout = stdout.split(';')[0]
            stdout = stdout.split('/')[0]

            return stdout
        except:
            return None
    return None


def args_parser():
    parser = ArgumentParser()

    parser.add_argument('--clear-cache', action='store_true')
    parser.add_argument('--info', default=None)
    parser.add_argument('--install', default=None)
    parser.add_argument('--list-demos', action='store_true')
    parser.add_argument('--list-apps', action='store_true')
    parser.add_argument('--run', default=None)

    parser.add_argument('-d', '--download', default=None)
    parser.add_argument('-i', '--image', default=None)
    parser.add_argument('-l', '--labels', default=None)
    parser.add_argument('-m', '--model', default=None)
    parser.add_argument('-r', '--res', default='hd')
    parser.add_argument('-f', '--video_fwk', default='v4l2')
    parser.add_argument('-v', '--video_src', default=None)

    parser.add_argument('-w', '--work_mode', default=None)
    parser.add_argument('-c', '--camera_params', default='200 60 60')

    return parser.parse_args()



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

from eiq.apps.switch_image.switch_image import eIQSwitchLabelImage
from eiq.apps.switch_video.switch_video import eIQVideoSwitchCore2
from eiq.modules.classification.classification_tflite import eIQObjectClassificationTFLite
from eiq.modules.detection.object_detection_ssd import eIQObjectDetection
from eiq.modules.detection.covid19_detection import eIQCOVID19Detection


APPS = {'switch_video': eIQVideoSwitchCore2}

DEMOS = {'covid19_detection': eIQCOVID19Detection,
         'object_classification_tflite': eIQObjectClassificationTFLite,
         'object_detection_tflite': eIQObjectDetection
         }



# Copyright 2021 NXP
# SPDX-License-Identifier: BSD-3-Clause

import os
from random import randint
from socket import gethostname
from stat import S_IEXEC
import sys
import threading

import gi
gi.require_versions({'GdkPixbuf': "2.0", 'Gtk': "3.0"})
from gi.repository.GdkPixbuf import Colorspace, Pixbuf
from gi.repository import GLib, Gtk

import cv2
import numpy as np

from eiq.apps.switch_image.config import SWITCH_IMAGE
from eiq.apps.utils import create_link, run_label_image
from eiq.config import BASE_DIR, ZIP
from eiq.utils import args_parser, check_data, Downloader


class eIQSwitchLabelImage(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self, title=SWITCH_IMAGE['window_title'])
        self.set_default_size(1280, 720)
        self.set_position(Gtk.WindowPosition.CENTER)
        self.args = args_parser()
        self.data = SWITCH_IMAGE
        self.msg = self.data['msg']
        self.base_dir = os.path.join(BASE_DIR, self.__class__.__name__)
        self.bin_dir = os.path.join(self.base_dir, "bin")
        self.media_dir = os.path.join(self.base_dir, "media")
        self.model_dir = os.path.join(self.base_dir, "model")

        self.binary = os.path.join(self.bin_dir, self.data['bin'])
        self.model = os.path.join(self.model_dir, self.data['model'])
        self.labels = os.path.join(self.model_dir, self.data['labels'])

        grid = Gtk.Grid(row_spacing=10, column_spacing=10, border_width=18)
        self.add(grid)
        self.hw_accel = self.get_hw_accel()
        self.value_returned = []
        self.label_returned = []
        self.value_returned_box = []
        self.label_returned_box = []
        self.displayed_image = Gtk.Image()
        self.image_map = Gtk.ListStore(str)

        download = Downloader(self.args)
        if not check_data(os.path.join(self.base_dir,
                                       self.__class__.__name__ + ZIP),
                          self.data['sha1'], self.binary, self.model,
                          self.labels):
            download.retrieve_data(self.data['src'],
                                   self.__class__.__name__ + ZIP,
                                   self.base_dir, self.data['sha1'], True)
        os.chmod(self.binary, S_IEXEC)
        create_link()

        self.get_bmp_images()

        grid.set_column_homogeneous(True)
        grid.set_row_homogeneous(True)

        model_box = Gtk.Box()
        model_name_box = Gtk.Box()
        result_box = Gtk.Box()
        percentage_box = Gtk.Box()
        inference_box = Gtk.Box()
        inference_value_box = Gtk.Box()
        image_label_box = Gtk.Box()
        image_map_box = Gtk.Box()
        image_box = Gtk.Box()

        self.image_combo_box = Gtk.ComboBox.new_with_model(self.image_map)
        self.image_combo_box.connect("changed", self.on_combo_image_changed)
        image_rendered_list = Gtk.CellRendererText()
        self.image_combo_box.pack_start(image_rendered_list, True)
        self.image_combo_box.add_attribute(image_rendered_list, "text", 0)

        for i in range(5):
            self.value_returned.append(Gtk.Entry())
            self.label_returned.append(Gtk.Label())
            self.value_returned_box.append(Gtk.Box())
            self.label_returned_box.append(Gtk.Box())

        self.image = self.get_random_image()
        self.set_displayed_image(self.image)
        self.set_initial_entrys()

        model_label = Gtk.Label()
        model_label.set_markup(self.msg['model_name'])
        self.model_name_label = Gtk.Label.new(None)
        result_label = Gtk.Label()
        result_label.set_markup(self.msg['labels'])
        percentage_label = Gtk.Label()
        percentage_label.set_markup(self.msg['confidence'])
        inference_label = Gtk.Label()
        inference_label.set_markup(self.msg['inf_time'])
        self.inference_value_label = Gtk.Label.new(None)
        image_label = Gtk.Label()
        image_label.set_markup(self.msg['select_img'])
        image_label.set_xalign(0.0)

        model_box.pack_start(model_label, True, True, 0)
        model_name_box.pack_start(self.model_name_label, True, True, 0)
        result_box.pack_start(result_label, True, True, 0)
        percentage_box.pack_start(percentage_label, True, True, 0)
        inference_box.pack_start(inference_label, True, True, 0)
        inference_value_box.pack_start(self.inference_value_label, True, True, 0)
        image_label_box.pack_start(image_label, True, True, 0)
        image_map_box.pack_start(self.image_combo_box, True, True, 0)

        for i in range(5):
            self.label_returned_box[i].pack_start(self.label_returned[i], True, True, 0)
            self.value_returned_box[i].pack_start(self.value_returned[i], True, True, 0)

        image_box.pack_start(self.displayed_image, True, True, 0)

        self.cpu_button = Gtk.Button.new_with_label("CPU")
        self.cpu_button.connect("clicked", self.run_cpu_inference)
        grid.attach(self.cpu_button, 3, 0, 1, 1)
        self.npu_button = Gtk.Button.new_with_label(self.hw_accel)
        self.npu_button.connect("clicked", self.run_npu_inference)
        grid.attach(self.npu_button, 4, 0, 1, 1)

        grid.attach(model_box, 0, 5, 2, 1)
        grid.attach(model_name_box, 0, 6, 2, 1)
        grid.attach(inference_box, 0, 7, 2, 1)
        grid.attach(inference_value_box, 0, 8, 2, 1)
        grid.attach(result_box, 6, 3, 1, 1)
        grid.attach(percentage_box, 7, 3, 1, 1)

        for i in range(5):
            grid.attach(self.label_returned_box[i], 6, (4+i), 1, 1)
            grid.attach(self.value_returned_box[i], 7, (4+i), 1, 1)

        grid.attach(image_label_box, 0, 2, 2, 1)
        grid.attach(image_map_box, 0, 3, 2, 1)
        grid.attach(image_box, 2, 1, 4, 10)

    def set_displayed_image(self, image):
        img = cv2.imread(image)
        img = cv2.resize(img, (640, 480))
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
        height, width = img.shape[:2]
        arr = np.ndarray.tostring(img)
        pixbuf = Pixbuf.new_from_data(arr, Colorspace.RGB, False, 8, width,
                                      height, width * 3, None, None)
        self.displayed_image.set_from_pixbuf(pixbuf)

    def on_combo_image_changed(self, combo):
        iterr = combo.get_active_iter()
        if iterr is not None:
            model = combo.get_model()
            image_name = model[iterr][0]
            print("Selected image: {}".format(image_name))
            self.image = os.path.join(self.media_dir, image_name)
            self.set_displayed_image(self.image)

    def get_bmp_images(self):
        if not os.path.exists(self.media_dir) or not os.listdir(self.media_dir):
            sys.exit(f"{self.media_dir} does not exists, try to run"
                     " 'pyeiq --clear-cache' and run this application again.")

        for file in os.listdir(self.media_dir):
            if file != "LICENSE.md":
                self.image_map.append([file])

    def get_random_image(self):
        images = []

        for file in os.listdir(self.media_dir):
            file = os.path.join(self.media_dir, file)
            images.append(file)

        index = randint(0, (len(images) - 1))
        return images[index]

    @staticmethod
    def get_hw_accel():
        if gethostname() == "imx8mpevk":
            return "NPU"

        return "GPU"

    def set_initial_entrys(self):
        for i in range(5):
            self.label_returned[i].set_text("")
            self.value_returned[i].set_editable(False)
            self.value_returned[i].set_can_focus(False)
            self.value_returned[i].set_text("0%")
            self.value_returned[i].set_alignment(xalign=0)
            self.value_returned[i].set_progress_fraction(-1)

    def set_returned_entrys(self, value):
        x = 0
        for i in value[2:]:
            self.label_returned[x].set_text(str(i[2]))
            self.value_returned[x].set_progress_fraction(float(i[0]))
            self.value_returned[x].set_text("{0:.2f}%".format(float(i[0])*100))
            x = x + 1

    def set_pre_inference(self):
        self.set_initial_entrys()
        self.model_name_label .set_text("")
        self.inference_value_label.set_text("Running...")
        self.image_combo_box.set_sensitive(False)
        self.cpu_button.set_sensitive(False)
        self.npu_button.set_sensitive(False)

    def set_post_inference(self, x):
        self.model_name_label.set_text(x[0])
        self.inference_value_label.set_text("{0:.2f} ms".format(float(x[1])))
        self.set_returned_entrys(x)
        self.image_combo_box.set_sensitive(True)
        self.cpu_button.set_sensitive(True)
        self.npu_button.set_sensitive(True)

        for i in x:
            print(i)

    def run_cpu_inference(self, window):
        self.set_pre_inference()
        thread = threading.Thread(target=self.run_inference, args=(False,))
        thread.daemon = True
        thread.start()

    def run_inference(self, accel):
        if accel:
            print("Running Inference on {0}".format(self.hw_accel))
        else:
            print("Running Inference on CPU")

        x = run_label_image(accel, self.binary, self.model,
                            self.image, self.labels)

        GLib.idle_add(self.set_post_inference, x)

    def run_npu_inference(self, window):
        self.set_pre_inference()
        thread = threading.Thread(target=self.run_inference, args=(True,))
        thread.daemon = True
        thread.start()

    @staticmethod
    def description():
        return ("This demo uses:\n   * TensorFlow Lite as inference engine."
                "\n   * CNN as default algorithm.\n\n"
                "This application offers a graphical interface for users to run\n"
                "an object classification demo using either CPU or GPU/NPU to\n"
                "perform inference on a list of available images.")

    @staticmethod
    def run():
        app = eIQSwitchLabelImage()
        app.connect("destroy", Gtk.main_quit)
        app.show_all()
        Gtk.main()
